@@ -334,6 +334,13 @@ zoom_changed_cb (WebKitWebView *web_view,
   g_free (address);
 }
 
+static void
+ephy_embed_history_cleared_cb (EphyHistory *history,
+                               EphyEmbed *embed)
+{
+  ephy_web_view_clear_history (EPHY_WEB_VIEW (embed->priv->web_view));
+}
+
 static void
 ephy_embed_grab_focus (GtkWidget *widget)
 {
@@ -360,6 +367,10 @@ ephy_embed_finalize (GObject *object)
   }
   g_slist_free (embed->priv->destroy_on_transition_list);
 
+  g_signal_handlers_disconnect_by_func (embed->priv->history,
+                                        ephy_embed_history_cleared_cb,
+                                        embed);
+
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
 
@@ -733,6 +744,7 @@ confirm_action_from_mime (WebKitWebView *web_view,
                                                 "it could potentially damage your documents or "
                                                 "invade your privacy. You can download it instead."),
                                               mime_description, suggested_filename);
+    action_label = STOCK_DOWNLOAD;
   } else if (action == DOWNLOAD_ACTION_OPEN && helper_app) {
     dialog = gtk_message_dialog_new (GTK_WINDOW (parent_window),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -955,6 +967,10 @@ ephy_embed_constructed (GObject *object)
   ephy_embed_prefs_add_embed (embed);
 
   embed->priv->history = EPHY_HISTORY (ephy_embed_shell_get_global_history (ephy_embed_shell_get_default ()));
+
+  g_signal_connect (embed->priv->history,
+                    "cleared", G_CALLBACK (ephy_embed_history_cleared_cb),
+                    embed);
 }
 
 static void
