@@ -30,6 +30,8 @@
 #include "ephy-location-entry.h"
 #include "ephy-location-action.h"
 #include "ephy-navigation-action.h"
+#include "ephy-navigation-history-action.h"
+#include "ephy-navigation-up-action.h"
 #include "ephy-topic-action.h"
 #include "ephy-zoom-action.h"
 #include "ephy-spinner-tool-item.h"
@@ -61,11 +63,6 @@ enum
 	LAST_ACTION
 };
 
-enum
-{
-	SENS_FLAG = 1 << 0
-};
-
 #define EPHY_TOOLBAR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOOLBAR, EphyToolbarPrivate))
 
 struct _EphyToolbarPrivate
@@ -135,6 +132,7 @@ ephy_toolbar_update_fixed_visibility (EphyToolbar *toolbar)
 
 	show = priv->leave_fullscreen_visible;
 	g_object_set (priv->sep_item, "visible", show, NULL);
+	g_object_set (priv->exit_button, "visible", show, NULL);
 	g_object_set (priv->fixed_toolbar, "visible", show, NULL);
 }
 
@@ -225,7 +223,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 	g_object_unref (priv->action_group);
 
 	action = priv->actions[BACK_ACTION] =
-		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
+		g_object_new (EPHY_TYPE_NAVIGATION_HISTORY_ACTION,
 			      "name", "NavigationBack",
 			      "label", _("_Back"),
 			      "stock_id", GTK_STOCK_GO_BACK,
@@ -235,7 +233,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			       */
 			      "arrow-tooltip", _("Back history"),
 			      "window", priv->window,
-			      "direction", EPHY_NAVIGATION_DIRECTION_BACK,
+			      "direction", EPHY_NAVIGATION_HISTORY_DIRECTION_BACK,
 			      "is_important", TRUE,
 			      NULL);
 	g_signal_connect_swapped (action, "open-link",
@@ -245,7 +243,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 	g_object_unref (action);
 
 	action = priv->actions[FORWARD_ACTION] =
-		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
+		g_object_new (EPHY_TYPE_NAVIGATION_HISTORY_ACTION,
 			      "name", "NavigationForward",
 			      "label", _("_Forward"),
 			      "stock_id", GTK_STOCK_GO_FORWARD,
@@ -255,7 +253,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			       */
 			      "arrow-tooltip", _("Forward history"),
 			      "window", priv->window,
-			      "direction", EPHY_NAVIGATION_DIRECTION_FORWARD,
+			      "direction", EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD,
 			      NULL);
 	g_signal_connect_swapped (action, "open-link",
 				  G_CALLBACK (ephy_link_open), toolbar);
@@ -264,7 +262,7 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 	g_object_unref (action);
 
 	action = priv->actions[UP_ACTION] =
-		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
+		g_object_new (EPHY_TYPE_NAVIGATION_UP_ACTION,
 			      "name", "NavigationUp",
 			      "label", _("_Up"),
 			      "stock_id", GTK_STOCK_GO_UP,
@@ -274,7 +272,6 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			       */
 			      "arrow-tooltip", _("List of upper levels"),
 			      "window", priv->window,
-			      "direction", EPHY_NAVIGATION_DIRECTION_UP,
 			      NULL);
 	g_signal_connect_swapped (action, "open-link",
 				  G_CALLBACK (ephy_link_open), toolbar);
